<?xml version="1.0" encoding="utf-8" ?>

<!--  Copyright 2010,2013 EUROPEAN UNION
  Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
  the European Commission - subsequent versions of the EUPL (the "Licence");
  You may not use this work except in compliance with the Licence.
  You may obtain a copy of the Licence at:
 
  http://ec.europa.eu/idabc/eupl
 
  Unless required by applicable law or agreed to in writing, software
  distributed under the Licence is distributed on an "AS IS" basis,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the Licence for the specific language governing permissions and
  limitations under the Licence.
 
  Date: 14/06/2013
  Authors: European Commission, Joint Research Centre
  Daniele Francioli, Emanuela Epure
  inspire-registry-info@jrc.ec.europa.eu
 
This work was supported by the EU  Interoperability Solutions for
European Public Administrations Programme (http://ec.europa.eu/isa)
through Action 1.17: Re-usable INSPIRE Reference Platform 
(http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).-->

<xsl:transform
    xmlns:xsl     = "http://www.w3.org/1999/XSL/Transform"
    xmlns:xsd     = "http://www.w3.org/2001/XMLSchema#"
    xmlns:rdf     = "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:rdfs    = "http://www.w3.org/2000/01/rdf-schema#"
    xmlns:owl     = "http://www.w3.org/2002/07/owl#"
    xmlns:dct     = "http://purl.org/dc/terms/"
    xmlns:foaf    = "http://xmlns.com/foaf/0.1/"
    xmlns:sioc    = "http://rdfs.org/sioc/ns#"
    xmlns:schema  = "http://schema.org/"
    xmlns:cc      = "http://creativecommons.org/ns#"
    xmlns:skos    = "http://www.w3.org/2004/02/skos/core#"
    xmlns:dcat    = "http://www.w3.org/ns/dcat#"
    xmlns:xhv     = "http://www.w3.org/1999/xhtml/vocab#"
    xmlns         = "http://www.w3.org/1999/xhtml"
    xmlns:vann    = "http://purl.org/vocab/vann/"
    xmlns:vs      = "http://www.w3.org/2003/06/sw-vocab-status/ns#"
    xmlns:rad     = "http://www.example.org/ns/radion#"
    xmlns:adms    = "http://www.example.org/ns/adms#"
    xmlns:wdrs    = "http://www.w3.org/2007/05/powder-s#"
    
xmlns:xs="http://www.w3.org/2001/XMLSchema" 
xmlns:ad="urn:x-inspire:specification:gmlas:Addresses:3.0" 
xmlns:au="urn:x-inspire:specification:gmlas:AdministrativeUnits:3.0" 
xmlns:base="urn:x-inspire:specification:gmlas:BaseTypes:3.2" 
xmlns:bui="urn:x-inspire:specification:gmlas:Buildings:0.0" 
xmlns:cp="urn:x-inspire:specification:gmlas:CadastralParcels:3.0" 
xmlns:gml="http://www.opengis.net/gml/3.2" 
xmlns:gn="urn:x-inspire:specification:gmlas:GeographicalNames:3.0" 
xmlns:tn="urn:x-inspire:specification:gmlas:CommonTransportElements:3.0" 
    
    
  	version="1.0">

  <xsl:output method="xml"
              media-type="application/rdf+xml"
              encoding="utf-8"
              indent="yes" />
              
  <xsl:param name="xsd">http://www.w3.org/2001/XMLSchema#</xsl:param>
  <xsl:param name="rdf">http://www.w3.org/1999/02/22-rdf-syntax-ns#</xsl:param>
  <xsl:param name="bns">http://inspire.ec.europa.eu/def/</xsl:param>
  
  <xsl:param name="status1">http://www.iso.org/19135/valid</xsl:param>

  <xsl:template match="/">
    <rdf:RDF>
      <xsl:call-template name="registers"/>
      <xsl:call-template name="register-items"/>
      <xsl:call-template name="register-item-values"/>
    </rdf:RDF>
  </xsl:template>
  
<!-- Registers -->  

  <xsl:template name="registers">
<!-- Node name to be replaced with the final one - note that the current one is mispelled -->  
    <xsl:for-each select="//register">
      <xsl:variable name="reg-uri" select="concat($bns,uri_name)"/>
      <rad:Repository rdf:about="{$reg-uri}">
        <dct:created rdf:datatype="{$xsd}dateTime"><xsl:value-of select="datecreation"/></dct:created>
        <xsl:for-each select="label | definition | description">
          <xsl:choose>
            <xsl:when test="local-name() = 'label' and text() != ''">
              <rdfs:label xml:lang="{@language}"><xsl:value-of select="."/></rdfs:label>
            </xsl:when>
            <xsl:when test="local-name() = 'definition' and text() != ''">
              <rdfs:comment xml:lang="{@language}"><xsl:value-of select="."/></rdfs:comment>
            </xsl:when>
            <xsl:when test="local-name() = 'description' and text() != ''">
              <dct:description xml:lang="{@language}"><xsl:value-of select="."/></dct:description>
            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
<!-- Node name to be replaced with the final one -->  
        <xsl:for-each select="items/item">
          <dct:hasPart rdf:resource="{$reg-uri}/{uri_name}"/>
        </xsl:for-each>
      </rad:Repository>
    </xsl:for-each>
  </xsl:template>

<!-- Register items -->  

  <xsl:template name="register-items">
<!-- Node name to be replaced with the final one -->  
    <xsl:for-each select="/item | //items/item">
      <xsl:variable name="reg-uri">
        <xsl:choose>
          <xsl:when test="itemclass/uri_name = 'CodeListItem'">
            <xsl:value-of select="concat($bns,'codelist','/')"/>
          </xsl:when>
        </xsl:choose>
      </xsl:variable>
      <xsl:variable name="regitem-uri" select="concat($reg-uri,uri_name)"/>
      <skos:ConceptScheme rdf:about="{$regitem-uri}">
        <dct:identifier rdf:datatype="{$xsd}string"><xsl:value-of select="uri_name"/></dct:identifier>
        <dct:created rdf:datatype="{$xsd}dateTime"><xsl:value-of select="datecreation"/></dct:created>
        <xsl:for-each select="label | definition | description">
          <xsl:choose>
            <xsl:when test="local-name() = 'label' and text() != ''">
              <skos:prefLabel xml:lang="{@language}"><xsl:value-of select="."/></skos:prefLabel>
            </xsl:when>
            <xsl:when test="local-name() = 'definition' and text() != ''">
              <skos:definition xml:lang="{@language}"><xsl:value-of select="."/></skos:definition>
            </xsl:when>
            <xsl:when test="local-name() = 'description' and text() != ''">
              <vann:usageNote xml:lang="{@language}"><xsl:value-of select="."/></vann:usageNote>
            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
        <xsl:call-template name="status"/>
<!-- Node name to be replaced with the final one -->  
        <xsl:if test="local-name(../../.) = 'register'">
          <dct:isPartOf rdf:resource="{$reg-uri}"/>
        </xsl:if>
        <xsl:call-template name="custom-attributes"/>
        <xsl:for-each select="collection/item">
          <skos:hasTopConcept rdf:resource="{$regitem-uri}/{uri_name}"/>
        </xsl:for-each>
      </skos:ConceptScheme>
    </xsl:for-each>
  </xsl:template>
  
<!-- Register item values -->  

  <xsl:template name="register-item-values">
<!-- Node name to be replaced with the final one -->  
    <xsl:for-each select="//item/collection/item">
      <xsl:variable name="reg-uri">
        <xsl:choose>
          <xsl:when test="itemclass/uri_name = 'CodeListValue'">
            <xsl:value-of select="concat($bns,'codelist/')"/>
          </xsl:when>
        </xsl:choose>
      </xsl:variable>
      <xsl:variable name="regitem-uri">
        <xsl:value-of select="concat($reg-uri,../../uri_name)"/>
      </xsl:variable>
      <skos:Concept rdf:about="{@id}">
        <dct:identifier rdf:datatype="{$xsd}string"><xsl:value-of select="uri_name"/></dct:identifier>
        <skos:notation rdf:datatype="{$xsd}string"><xsl:value-of select="uri_name"/></skos:notation>
        <xsl:for-each select="label | definition | description">
          <xsl:choose>
            <xsl:when test="local-name() = 'label' and text() != ''">
              <skos:prefLabel xml:lang="{@language}"><xsl:value-of select="."/></skos:prefLabel>
            </xsl:when>
            <xsl:when test="local-name() = 'definition' and text() != ''">
              <skos:definition xml:lang="{@language}"><xsl:value-of select="."/></skos:definition>
            </xsl:when>
            <xsl:when test="local-name() = 'description' and text() != ''">
              <vann:usageNote xml:lang="{@language}"><xsl:value-of select="."/></vann:usageNote>
            </xsl:when>
          </xsl:choose>
        </xsl:for-each>
        <xsl:call-template name="status"/>
<!-- Node name to be replaced with the final one -->  
        <xsl:if test="../../itemclass/uri_name/text() = 'CodeListItem'">
          <skos:inScheme rdf:resource="{$regitem-uri}"/>
          <skos:topConceptOf rdf:resource="{$regitem-uri}"/>
        </xsl:if>
<!-- Node name to be replaced with the final one -->  
        <xsl:if test="local-name(../../../../.) = 'register'">
          <dct:isPartOf rdf:resource="{$reg-uri}"/>
        </xsl:if>
        <xsl:call-template name="custom-attributes"/>
      </skos:Concept>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="status">
    <xsl:for-each select="status">
        <xsl:variable name="status-code">
          <xsl:choose>
            <xsl:when test="uuid = 1">
              <xsl:value-of select="$status1"/>
            </xsl:when>
<!--

To be completed based on the possible values of attribute status/@id

-->
          </xsl:choose>
        </xsl:variable>
        <xsl:variable name="status-label">
          <xsl:choose>
            <xsl:when test="uuid = 1">
              <xsl:text>stable</xsl:text>
            </xsl:when>
<!--

To be completed based on the possible values of attribute status/@id

-->
          </xsl:choose>
        </xsl:variable>
        <vs:terms_status><xsl:value-of select="$status-label"/></vs:terms_status>
<!-- What this will be? Free text or keywords? Should the language be specified? -->        
        <vs:moreinfo><xsl:value-of select="label"/></vs:moreinfo>
<!-- Another way to specify the status, by using the ISO code list  -->        
        <adms:status rdf:resource="{$status-code}"/>
<!-- Another way to specify the status, using the ADMS code list / taxonomy -->        
        <xsl:variable name="adms-status">
          <xsl:choose>
<!--
            <xsl:when test="status/@id = 'http://www.iso.org/19135/notValid'">
              <xsl:text>http://purl.org/adms/status/??</xsl:text>
            </xsl:when>
-->            
            <xsl:when test="uuid = 1">
              <xsl:text>http://purl.org/adms/status/Completed</xsl:text>
            </xsl:when>
<!--

To be completed based on the possible value of attribute status/@id

                    -->
                    <!--            
                                <xsl:when test="status/uuid = ??">
                                  <xsl:text>http://purl.org/adms/status/WithDrawn</xsl:text>
                                </xsl:when>
                                <xsl:when test="status/uuid = ??">
                                  <xsl:text>http://purl.org/adms/status/Deprecated</xsl:text>
                                </xsl:when>
                                <xsl:when test="status/uuid = ??">
                                  <xsl:text>http://purl.org/adms/status/UnderDevelopment</xsl:text>
                                </xsl:when>
                    -->            
                </xsl:choose>
            </xsl:variable>
            <adms:status rdf:resource="{$adms-status}"/>
        </xsl:for-each>
    </xsl:template>
  
    <xsl:template name="custom-attributes">
        <xsl:for-each select="customattributes/customattribute">
            <xsl:choose>
                <xsl:when test="name = 'ApplicationSchema'">
                    <dct:source>
                        <rdf:Description rdf:about="{$bns}{label}">
                            <!-- The text here is always in English? -->          
                            <rdfs:label xml:lang="en">
                                <xsl:value-of select="label"/>
                            </rdfs:label>
                        </rdf:Description>
                    </dct:source>
                </xsl:when>
            </xsl:choose>
        </xsl:for-each>
    </xsl:template>

</xsl:transform>
