/*
* Copyright 2010,2013 EUROPEAN UNION
* Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* http://ec.europa.eu/idabc/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* Date: 06/02/2014
* Authors: European Commission, Joint Research Centre
* Daniele Francioli, Emanuela Epure
* inspire-registry-info@jrc.ec.europa.eu
*
* This work was supported by the EU  Interoperability Solutions for
* European Public Administrations Programme (http://ec.europa.eu/isa)
* through Action 1.17: Re-usable INSPIRE Reference Platform 
* (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
*/

package eu.europa.ec.re3gistry.model;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "customattribute")
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Customattribute.findAll", query = "SELECT c FROM Customattribute c"),
    @NamedQuery(name = "Customattribute.findByUuid", query = "SELECT c FROM Customattribute c WHERE c.uuid = :uuid"),
    @NamedQuery(name = "Customattribute.findByName", query = "SELECT c FROM Customattribute c WHERE c.name = :name"),
    @NamedQuery(name = "Customattribute.findByRequired", query = "SELECT c FROM Customattribute c WHERE c.required = :required"),
    @NamedQuery(name = "Customattribute.findByMultivalue", query = "SELECT c FROM Customattribute c WHERE c.multivalue = :multivalue"),
    @NamedQuery(name = "Customattribute.findByCoded", query = "SELECT c FROM Customattribute c WHERE c.coded = :coded"),
    @NamedQuery(name = "Customattribute.findByIsforeignkey", query = "SELECT c FROM Customattribute c WHERE c.isforeignkey = :isforeignkey"),
    @NamedQuery(name = "Customattribute.findByDatecreation", query = "SELECT c FROM Customattribute c WHERE c.datecreation = :datecreation"),
    @NamedQuery(name = "Customattribute.findByDatelastupdate", query = "SELECT c FROM Customattribute c WHERE c.datelastupdate = :datelastupdate")})
public class Customattribute implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @Column(name = "uuid")
    private String uuid;
    @Basic(optional = false)
    @Column(name = "name")
    private String name;
    @Basic(optional = false)
    @Column(name = "required")
    private boolean required;
    @Basic(optional = false)
    @Column(name = "multivalue")
    private boolean multivalue;
    @Basic(optional = false)
    @Column(name = "coded")
    private boolean coded;
    @Basic(optional = false)
    @Column(name = "isforeignkey")
    private boolean isforeignkey;       
    @JoinColumn(name = "itemclass_id", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Itemclass itemclass;
    @Basic(optional = false)
    @Column(name = "datecreation")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datecreation;
    @Column(name = "datelastupdate")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datelastupdate;

    public Customattribute() {
    }

    public Customattribute(String uuid) {
        this.uuid = uuid;
    }

    public Customattribute(String uuid, String name, boolean required, boolean multivalue, boolean coded, boolean isforeignkey, Date datecreation) {
        this.uuid = uuid;
        this.name = name;
        this.required = required;
        this.multivalue = multivalue;
        this.coded = coded;
        this.isforeignkey = isforeignkey;
        this.datecreation = datecreation;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public boolean getRequired() {
        return required;
    }

    public void setRequired(boolean required) {
        this.required = required;
    }

    public boolean getMultivalue() {
        return multivalue;
    }

    public void setMultivalue(boolean multivalue) {
        this.multivalue = multivalue;
    }

    public boolean getCoded() {
        return coded;
    }

    public void setCoded(boolean coded) {
        this.coded = coded;
    }

    public boolean getIsforeignkey() {
        return isforeignkey;
    }

    public void setIsforeignkey(boolean isforeignkey) {
        this.isforeignkey = isforeignkey;
    }

    public Itemclass getItemclass() {
        return itemclass;
    }

    public void setItemclass(Itemclass itemclass) {
        this.itemclass = itemclass;
    }
    
    public Date getDatecreation() {
        return datecreation;
    }

    public void setDatecreation(Date datecreation) {
        this.datecreation = datecreation;
    }

    public Date getDatelastupdate() {
        return datelastupdate;
    }

    public void setDatelastupdate(Date datelastupdate) {
        this.datelastupdate = datelastupdate;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (uuid != null ? uuid.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Customattribute)) {
            return false;
        }
        Customattribute other = (Customattribute) object;
        if ((this.uuid == null && other.uuid != null) || (this.uuid != null && !this.uuid.equals(other.uuid))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "eu.europa.ec.re3gistry.model.Customattribute[ uuid=" + uuid + " ]";
    }
}
