/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 06/02/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.model;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "item")
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Item.findAll", query = "SELECT i FROM Item i"),
    @NamedQuery(name = "Item.findByUuid", query = "SELECT i FROM Item i WHERE i.uuid = :uuid"),
    @NamedQuery(name = "Item.findByUriName", query = "SELECT i FROM Item i WHERE i.uriName = :uriName"),
    @NamedQuery(name = "Item.findByDatecreation", query = "SELECT i FROM Item i WHERE i.datecreation = :datecreation"),
    @NamedQuery(name = "Item.findByDatelastupdate", query = "SELECT i FROM Item i WHERE i.datelastupdate = :datelastupdate"),
    //custom
    @NamedQuery(name = "Item.findByRegisterItemclass", query = "SELECT i FROM Item i WHERE i.register = :register AND i.itemclass = :itemclass"),
})
public class Item implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @Column(name = "uuid")
    private String uuid;
    @Basic(optional = false)
    @Column(name = "uri_name")
    private String uriName;
    @Basic(optional = false)
    @Column(name = "datecreation")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datecreation;
    @Column(name = "datelastupdate")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datelastupdate;
    @Column(name = "datetimeaddition")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datetimeaddition;
    @Column(name = "datetimesupersession")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datetimesupersession;
    @Column(name = "datetimeretirment")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datetimeretirment;
    @Column(name = "datetimeinvalidation")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datetimeinvalidation;
    @JoinColumn(name = "status_id", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Status status;
    @JoinColumn(name = "register_id", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Register register;
    @JoinColumn(name = "itemclass_id", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Itemclass itemclass;
    @JoinColumn(name = "successor_id", referencedColumnName = "uuid")
    @ManyToOne
    private Item successor;
    @JoinColumn(name = "predecessor_id", referencedColumnName = "uuid")
    @ManyToOne
    private Item predecessor;
    @JoinColumn(name = "parent_id", referencedColumnName = "uuid")
    @ManyToOne
    private Item parent;
    @JoinColumn(name = "collection_id", referencedColumnName = "uuid")
    @ManyToOne
    private Item collection;
    @JoinColumn(name = "externalreference_id", referencedColumnName = "uuid")
    @ManyToOne
    private ExternalReference externalreference;

    public Item() {
    }

    public Item(String uuid) {
        this.uuid = uuid;
    }

    public Item(String uuid, String uriName, Date datecreation) {
        this.uuid = uuid;
        this.uriName = uriName;
        this.datecreation = datecreation;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getUriName() {
        return uriName;
    }

    public void setUriName(String uriName) {
        this.uriName = uriName;
    }

    public Date getDatecreation() {
        return datecreation;
    }

    public void setDatecreation(Date datecreation) {
        this.datecreation = datecreation;
    }

    public Date getDatelastupdate() {
        return datelastupdate;
    }

    public void setDatelastupdate(Date datelastupdate) {
        this.datelastupdate = datelastupdate;
    }

    public Date getDatetimeaddition() {
        return datetimeaddition;
    }

    public void setDatetimeaddition(Date datetimeaddition) {
        this.datetimeaddition = datetimeaddition;
    }

    public Date getDatetimesupersession() {
        return datetimesupersession;
    }

    public void setDatetimesupersession(Date datetimesupersession) {
        this.datetimesupersession = datetimesupersession;
    }

    public Date getDatetimeretirment() {
        return datetimeretirment;
    }

    public void setDatetimeretirment(Date datetimeretirment) {
        this.datetimeretirment = datetimeretirment;
    }

    public Date getDatetimeinvalidation() {
        return datetimeinvalidation;
    }

    public void setDatetimeinvalidation(Date datetimeinvalidation) {
        this.datetimeinvalidation = datetimeinvalidation;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    public Register getRegister() {
        return register;
    }

    public void setRegister(Register register) {
        this.register = register;
    }

    public Itemclass getItemclass() {
        return itemclass;
    }

    public void setItemclass(Itemclass itemclass) {
        this.itemclass = itemclass;
    }

    public Item getSuccessor() {
        return successor;
    }

    public void setSuccessor(Item successor) {
        this.successor = successor;
    }

    public Item getPredecessor() {
        return predecessor;
    }

    public void setPredecessor(Item predecessor) {
        this.predecessor = predecessor;
    }

    public Item getParent() {
        return parent;
    }

    public void setParent(Item parent) {
        this.parent = parent;
    }

    public Item getCollection() {
        return collection;
    }

    public void setCollection(Item collection) {
        this.collection = collection;
    }

    public ExternalReference getExternalreference() {
        return externalreference;
    }

    public void setExternalreference(ExternalReference externalreference) {
        this.externalreference = externalreference;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (uuid != null ? uuid.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Item)) {
            return false;
        }
        Item other = (Item) object;
        if ((this.uuid == null && other.uuid != null) || (this.uuid != null && !this.uuid.equals(other.uuid))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "eu.europa.ec.re3gistry.model.Item[ uuid=" + uuid + " ]";
    }
    
}
