/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 06/02/2014
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.model;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.xml.bind.annotation.XmlRootElement;

@Entity
@Table(name = "localization")
@XmlRootElement
@NamedQueries({
    @NamedQuery(name = "Localization.findAll", query = "SELECT l FROM Localization l"),
    @NamedQuery(name = "Localization.findByUuid", query = "SELECT l FROM Localization l WHERE l.uuid = :uuid"),
    @NamedQuery(name = "Localization.findByLabel", query = "SELECT l FROM Localization l WHERE l.label = :label"),
    @NamedQuery(name = "Localization.findByDefinition", query = "SELECT l FROM Localization l WHERE l.definition = :definition"),
    @NamedQuery(name = "Localization.findByDescription", query = "SELECT l FROM Localization l WHERE l.description = :description"),
    @NamedQuery(name = "Localization.findByDatecreation", query = "SELECT l FROM Localization l WHERE l.datecreation = :datecreation"),
    @NamedQuery(name = "Localization.findByDatelastupdate", query = "SELECT l FROM Localization l WHERE l.datelastupdate = :datelastupdate"),
    @NamedQuery(name = "Localization.findByStatusId", query = "SELECT l FROM Localization l WHERE l.status = :status"),
    //custom
    @NamedQuery(name = "Localization.findByItemclass", query = "SELECT l FROM Localization l WHERE l.itemclass = :itemclass"),
    @NamedQuery(name = "Localization.findByRegister", query = "SELECT l FROM Localization l WHERE l.register = :register"),
    @NamedQuery(name = "Localization.findByItemLanguagecode", query = "SELECT l FROM Localization l WHERE l.item = :item AND l.language = :language"),
    @NamedQuery(name = "Localization.findByRegisterLanguage", query = "SELECT l FROM Localization l WHERE l.register = :register AND l.language = :language"),
    @NamedQuery(name = "Localization.findByItemclassLanguagecode", query = "SELECT l FROM Localization l WHERE l.itemclass = :itemclass AND l.language = :language")
})
public class Localization implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @Column(name = "uuid")
    private String uuid;
    @Basic(optional = false)
    @Column(name = "label")
    private String label;
    @Column(name = "definition")
    private String definition;
    @Column(name = "description")
    private String description;
    @Basic(optional = false)
    @Column(name = "datecreation")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datecreation;
    @Column(name = "datelastupdate")
    @Temporal(TemporalType.TIMESTAMP)
    private Date datelastupdate;
    @JoinColumn(name = "registry_id", referencedColumnName = "uuid")
    @ManyToOne
    private Registry registry;
    @JoinColumn(name = "register_id", referencedColumnName = "uuid")
    @ManyToOne
    private Register register;
    @JoinColumn(name = "language_id", referencedColumnName = "uuid")
    @ManyToOne(optional = false)
    private Languagecode language;
    @JoinColumn(name = "itemclass_id", referencedColumnName = "uuid")
    @ManyToOne
    private Itemclass itemclass;
    @JoinColumn(name = "item_id", referencedColumnName = "uuid")
    @ManyToOne
    private Item item;
    @JoinColumn(name = "customattributevalue_id", referencedColumnName = "uuid")
    @ManyToOne
    private Customattributevalue customattributevalue;
    @JoinColumn(name = "status_id", referencedColumnName = "uuid")
    @ManyToOne
    private Status status;

    public Localization() {
    }

    public Localization(String uuid) {
        this.uuid = uuid;
    }

    public Localization(String uuid, String label, Date datecreation) {
        this.uuid = uuid;
        this.label = label;
        this.datecreation = datecreation;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getDefinition() {
        return definition;
    }

    public void setDefinition(String definition) {
        this.definition = definition;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Date getDatecreation() {
        return datecreation;
    }

    public void setDatecreation(Date datecreation) {
        this.datecreation = datecreation;
    }

    public Date getDatelastupdate() {
        return datelastupdate;
    }

    public void setDatelastupdate(Date datelastupdate) {
        this.datelastupdate = datelastupdate;
    }

    public Registry getRegistry() {
        return registry;
    }

    public void setRegistry(Registry registry) {
        this.registry = registry;
    }

    public Register getRegister() {
        return register;
    }

    public void setRegister(Register register) {
        this.register = register;
    }

    public Languagecode getLanguage() {
        return language;
    }

    public void setLanguage(Languagecode language) {
        this.language = language;
    }

    public Itemclass getItemclass() {
        return itemclass;
    }

    public void setItemclass(Itemclass itemclass) {
        this.itemclass = itemclass;
    }

    public Item getItem() {
        return item;
    }

    public void setItem(Item item) {
        this.item = item;
    }

    public Customattributevalue getCustomattributevalue() {
        return customattributevalue;
    }

    public void setCustomattributevalue(Customattributevalue customattributevalue) {
        this.customattributevalue = customattributevalue;
    }

    public Status getStatus() {
        return status;
    }

    public void setStatus(Status status) {
        this.status = status;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (uuid != null ? uuid.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Localization)) {
            return false;
        }
        Localization other = (Localization) object;
        if ((this.uuid == null && other.uuid != null) || (this.uuid != null && !this.uuid.equals(other.uuid))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "eu.europa.ec.re3gistry.model.Localization[ uuid=" + uuid + " ]";
    }
}
