/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 14/06/2013
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.re3gistryexport.marshaller;

import eu.europa.ec.re3gistry.constants.EntityNames;
import eu.europa.ec.re3gistry.jaxbmodel.register.CollectionItemLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.ItemLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.RegisterLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.StatusLabel;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.CodelistAttribute;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.ExternalReferenceAttribute;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.LabelAttribute;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.RegisterAttribute;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.RegistryAttribute;
import eu.europa.ec.re3gistry.model.Customattributevalue;
import eu.europa.ec.re3gistry.model.ExternalReference;
import eu.europa.ec.re3gistry.model.Item;
import eu.europa.ec.re3gistry.model.Itemclass;
import eu.europa.ec.re3gistry.model.Languagecode;
import eu.europa.ec.re3gistry.model.Localization;
import eu.europa.ec.re3gistry.model.Register;
import eu.europa.ec.re3gistry.model.Registry;
import eu.europa.ec.re3gistry.model.Status;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

public class CodelistExport {

    private String folderSeparator = System.getProperty("file.separator");
    private final String folderExport;
    private final EntityManager em;
    private final List<Item> itemList;
    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(CodelistExport.class);
    private final RegisterLabel registerLabel;

    public CodelistExport(String folderExport, EntityManager em, List<Item> itemList, RegisterLabel registerLabel) {
        this.folderExport = folderExport;
        this.em = em;
        this.itemList = itemList;
        this.registerLabel = registerLabel;
    }

    public HashMap<String, ArrayList<ItemLabel>> getHash() {
        HashMap<String, ArrayList<ItemLabel>> hashItemLabel = new HashMap<String, ArrayList<ItemLabel>>();
        File exportFile = new File(folderExport);
        if (!exportFile.exists()) {
            exportFile.mkdir();
        }

        if (!itemList.isEmpty()) {
            for (Item item : itemList) {
                log.info("Start to export the uri_name: " + item.getUriName() + " --- item class: " + item.getItemclass().getUriName());
//                if (item.getItemclass().getUuid().equals("1") || item.getItemclass().getUuid().equals("5")) {
                    /**
                     * item details
                     */
                    String itemUUID = item.getUuid();
                    String itemUriName = item.getUriName();
                    Date itemDatecreation = item.getDatecreation();
                    Date itemDatelastupdate = item.getDatelastupdate();

                    log.info("Start to export the " + itemUriName);

                    Date itemDatetimeaddition = item.getDatetimeaddition();
                    Date itemDatetimesupersession = item.getDatetimesupersession();
                    Date itemDatetimeretirment = item.getDatetimeretirment();
                    Date itemDatetimeinvalidation = item.getDatetimeinvalidation();

                    Status itemStatus = item.getStatus();
                    Itemclass itemItemClass = item.getItemclass();
                    Item itemSuccessor = item.getSuccessor();
                    Register itemRegister = item.getRegister();
                    Item itemPredecessor = item.getPredecessor();
                    Item itemParent = item.getParent();
                    Item itemCollection = item.getCollection();
                    ExternalReference externalReference = item.getExternalreference();
//                    if (externalReference != null) {
                        /**
                         * create a new ItemLabel
                         */
                        String ITEM_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.item = :item";
                        Query qil = em.createQuery(ITEM_LOCALIZATION_QUERY);
                        qil.setParameter("item", item);

                        List<Localization> itemLocalizationList = qil.getResultList();
                        for (Localization localization : itemLocalizationList) {
                            /**
                             * create ItemLabel
                             */
                            ItemLabel itemLabel = new ItemLabel();
                            itemLabel.setUuid(itemUUID);
                            itemLabel.setUri_name(itemUriName);
                            itemLabel.setDatecreation(itemDatecreation);
                            itemLabel.setDatelastupdate(itemDatelastupdate);

                            itemLabel.setDatetimeaddition(itemDatetimeaddition);
                            itemLabel.setDatetimesupersession(itemDatetimesupersession);
                            itemLabel.setDatetimeretirment(itemDatetimeretirment);
                            itemLabel.setDatetimeinvalidation(itemDatetimeinvalidation);

                            String localizationUUID = localization.getUuid();
                            Item localizationItem = localization.getItem();
                            Itemclass localizationItemClass = localization.getItemclass();
                            Languagecode localizationLanguage = localization.getLanguage();

                            Register localizationRegister = localization.getRegister();
                            Customattributevalue localizationCustomattribute = localization.getCustomattributevalue();
                            String localizationLabel = localization.getLabel();

                            String localizationDefinition = localization.getDefinition();
                            String localizationDescription = localization.getDescription();
                            Date localizationDatecreation = localization.getDatecreation();
                            Date localizationDatelastupdate = localization.getDatelastupdate();
                            String localizationLanguageIsocode = localizationLanguage.getIsocode();

                            StatusLabel statusLabel = new StatusLabel();
                            statusLabel.setUuid(itemStatus.getUuid());
                            statusLabel.setUri(itemStatus.getUri());

                            String STATUS_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.language = :language AND loc.status = :status";
                            Query qstatus = em.createQuery(STATUS_LOCALIZATION_QUERY);
                            qstatus.setParameter("language", localizationLanguage);
                            qstatus.setParameter("status", itemStatus);

                            List<Localization> statusLocalizationList = qstatus.getResultList();
                            for (Localization statusLocalization : statusLocalizationList) {
                                statusLabel.setLabelAttribute(statusLocalization.getLabel(), localizationLanguageIsocode);
                            }

                            itemLabel.setStatus(statusLabel);
                            itemLabel.setLabel(localizationLabel, localizationLanguageIsocode);
                            itemLabel.setDefinition(localizationDefinition, localizationLanguageIsocode);
                            itemLabel.setDescription(localizationDescription, localizationLanguageIsocode);

                            /**
                             * set register
                             */
                            setRegister(itemRegister, em, localizationLanguage, itemLabel);

                            /**
                             * set the codelist if the collection is NOT NULL
                             */
                            setCodelist(itemCollection, em, localizationLanguage, itemLabel);

                            /**
                             * ITEMCLASS
                             */
                            ExportUtility.insertItemClass(itemItemClass, em, itemLabel, localizationLanguage);

                            /**
                             * PARENT
                             */
                            ExportUtility.insertParent(itemParent, em, itemLabel, localizationLanguage);

                            /**
                             * PREDECESSOR
                             */
                            ExportUtility.insertPredecessor(itemPredecessor, em, itemLabel);

                            /**
                             * SUCCESSOR
                             */
                            ExportUtility.insertSuccessor(itemSuccessor, em, itemLabel);

                            /**
                             * CUSTOM ATTRIBUTE
                             */
                            ExportUtility.insertCustomAttribute(em, item, itemLabel, localizationLanguage);

                            /**
                             * COLLECTION
                             */
                            File itemFile = insertCollection(item, exportFile, itemLabel, em, localizationLanguage, localizationLabel);

                            /**
                             * children
                             */
                            ExportUtility.insertChildren(em, item, itemLabel, localizationLanguage, localizationLabel);

                            /**
                             * external reference
                             */
                            insertExternalReferenceCodelist(externalReference, itemLabel, exportFile);

                            /**
                             * create XML
                             */
                            createXML(itemFile, itemLabel);

                            if (itemCollection == null) {
                                if (hashItemLabel.containsKey(localizationLanguageIsocode)) {
                                    hashItemLabel.get(localizationLanguageIsocode).add(itemLabel);
                                } else {
                                    ArrayList<ItemLabel> array = new ArrayList<ItemLabel>();
                                    array.add(itemLabel);
                                    hashItemLabel.put(localizationLanguageIsocode, array);
                                }
                            }
                        }
//                    } //external reference not null
                }
//            }//end if for itemclass 1
        }

        return hashItemLabel;
    }

    private File insertCollection(Item item, File exportFile, ItemLabel itemLabel, EntityManager em, Languagecode localizationLanguageItem, String localizationLabel) {
        Item itemCollection = item.getCollection();
        String itemFileName;
        File itemFile;
        /**
         * COLLECTION if the collection is not null we have a VALUE of the
         * codelist
         */
        if (itemCollection != null) {
            String itemCollectionUUID = itemCollection.getUuid();
            String itemCollectionUriName = itemCollection.getUriName();

            ItemLabel collectionItemLabel = new ItemLabel();
            collectionItemLabel.setUuid(itemCollectionUUID);
            collectionItemLabel.setUri_name(itemCollectionUriName);

            /**
             * create the VALUES folder
             */
            itemFileName = exportFile.getAbsolutePath().concat(folderSeparator + itemCollectionUriName);
            itemFile = new File(itemFileName);
            if (!itemFile.exists()) {
                itemFile.mkdir();
            }

            itemFileName = itemFile.getAbsolutePath().concat(folderSeparator + itemLabel.getUri_name());
            itemFile = new File(itemFileName);
            if (!itemFile.exists()) {
                itemFile.mkdir();
            }
        } else {
            /**
             * create the name for the item which is not a value
             */
            itemFileName = exportFile.getAbsolutePath().concat(folderSeparator + itemLabel.getUri_name());
            itemFile = new File(itemFileName);
            if (!itemFile.exists()) {
                itemFile.mkdir();
            }

            String COLLECTION_ITEMCLASS_QUERY = "SELECT it FROM " + EntityNames.ENTITY_ITEM + " it WHERE it.collection = :item";
            Query qcicl = em.createQuery(COLLECTION_ITEMCLASS_QUERY);
            qcicl.setParameter("item", item);
            List<Item> itemList = qcicl.getResultList();

            if (!itemList.isEmpty()) {
                ArrayList<ItemLabel> arrayListItemLabel = new ArrayList<ItemLabel>();
                for (Item childitem : itemList) {
                    String CHILDITEM_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.item = :childitem  AND loc.language = :language";
                    Query qrlc = em.createQuery(CHILDITEM_LOCALIZATION_QUERY);
                    qrlc.setParameter("childitem", childitem);
                    qrlc.setParameter("language", localizationLanguageItem);

                    List<Localization> childitemLocalizationList = qrlc.getResultList();
                    if (!childitemLocalizationList.isEmpty() && childitemLocalizationList.size() == 1) {
                        String childitemlanguageIsocode = childitemLocalizationList.get(0).getLanguage().getIsocode();

                        ItemLabel childItemLabel = new ItemLabel();

                        childItemLabel.setUuid(childitem.getUuid());
                        childItemLabel.setUri_name(childitem.getUriName());
                        childItemLabel.setDatecreation(childitem.getDatecreation());
                        childItemLabel.setDatelastupdate(childitem.getDatelastupdate());
                        childItemLabel.setDatetimeaddition(childitem.getDatetimeaddition());
                        childItemLabel.setDatetimesupersession(childitem.getDatetimesupersession());
                        childItemLabel.setDatetimeretirment(childitem.getDatetimeretirment());
                        childItemLabel.setDatetimeinvalidation(item.getDatetimeinvalidation());

                        /**
                         * PARENT
                         */
                        ExportUtility.insertParent(childitem.getParent(), em, childItemLabel, childitemLocalizationList.get(0).getLanguage());

                        StatusLabel statusLabel = new StatusLabel();
                        statusLabel.setUuid(childitem.getStatus().getUuid());
                        statusLabel.setUri(childitem.getStatus().getUri());

                        String STATUS_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.language = :language  AND loc.status = :status";
                        Query qstatus = em.createQuery(STATUS_LOCALIZATION_QUERY);
                        qstatus.setParameter("language", childitemLocalizationList.get(0).getLanguage());
                        qstatus.setParameter("status", childitem.getStatus());
                        List<Localization> statusLocalizationList = qstatus.getResultList();
                        for (Localization statusLocalization : statusLocalizationList) {
                            statusLabel.setLabelAttribute(statusLocalization.getLabel(), childitemlanguageIsocode);
                        }

                        childItemLabel.setStatus(statusLabel);
                        childItemLabel.setLabel(childitemLocalizationList.get(0).getLabel(), childitemlanguageIsocode);
                        childItemLabel.setDescription(childitemLocalizationList.get(0).getDescription(), childitemlanguageIsocode);
                        childItemLabel.setDefinition(childitemLocalizationList.get(0).getDefinition(), childitemlanguageIsocode);

                        /**
                         * set codelist
                         */
                        String codeListUriName = item.getUriName();
                        CodelistAttribute codelistAttribute = new CodelistAttribute();
                        codelistAttribute.setUri_name(codeListUriName);
                        LabelAttribute labelAttribute = new LabelAttribute();
                        labelAttribute.setValue(localizationLabel);
                        labelAttribute.setLanguage(localizationLanguageItem.getIsocode());
                        codelistAttribute.setLabel(labelAttribute);
                        childItemLabel.setCodelist(codelistAttribute);

                        Register register = item.getRegister();

                        String registerBaseUri = register.getBaseUri();
                        String registerUriName = register.getUriName();

                        RegisterAttribute registerAttribute = new RegisterAttribute();
                        registerAttribute.setBase_uri(registerBaseUri);
                        registerAttribute.setUri_name(registerUriName);

                        String REGISTER_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.register = :register AND loc.language = :language";
                        Query qrel = em.createQuery(REGISTER_LOCALIZATION_QUERY);
                        qrel.setParameter("register", item.getRegister());
                        qrel.setParameter("language", localizationLanguageItem);

                        List<Localization> localizationRegisterList = qrel.getResultList();
                        for (Localization registerlocalization : localizationRegisterList) {
                            LabelAttribute registerlabelAttribute = new LabelAttribute();
                            registerlabelAttribute.setValue(registerlocalization.getLabel());
                            registerlabelAttribute.setLanguage(registerlocalization.getLanguage().getIsocode());
                            registerAttribute.setLabel(registerlabelAttribute);
                        }

                        RegistryAttribute registryAttribute = new RegistryAttribute();
                        Registry registry = register.getRegistry();
                        registryAttribute.setBase_uri(registry.getBaseUri());
                        registryAttribute.setUri_name(registry.getUriName());

                        String REGISTER_LOCALIZATION = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.registry = :registry AND loc.language = :language";
                        Query qr = em.createQuery(REGISTER_LOCALIZATION);
                        qr.setParameter("registry", registry);
                        qr.setParameter("language", localizationLanguageItem);

                        List<Localization> registerLocalization = qr.getResultList();
                        for (Localization registerLocalizationItem : registerLocalization) {
                            String registerLocalizationItemLabel = registerLocalizationItem.getLabel();
                            Languagecode registerLocalizationItemLanguage = registerLocalizationItem.getLanguage();

                            LabelAttribute registerlabelAttribute = new LabelAttribute();
                            registerlabelAttribute.setValue(registerLocalizationItemLabel);
                            registerlabelAttribute.setLanguage(registerLocalizationItemLanguage.getIsocode());

                            registryAttribute.setLabel(registerlabelAttribute);
                        }

                        registerAttribute.setRegistry(registryAttribute);
                        /**
                         *
                         */
                        childItemLabel.setRegister(registerAttribute);

                        /**
                         * ITEMCLASS
                         */
                        ExportUtility.insertItemClass(childitem.getItemclass(), em, childItemLabel, childitemLocalizationList.get(0).getLanguage());

                        /**
                         * PARENT
                         */
                        ExportUtility.insertParent(childitem.getParent(), em, childItemLabel, childitemLocalizationList.get(0).getLanguage());

                        /**
                         * PREDECESSOR
                         */
                        ExportUtility.insertPredecessor(childitem.getPredecessor(), em, childItemLabel);

                        /**
                         * SUCCESSOR
                         */
                        ExportUtility.insertSuccessor(childitem.getSuccessor(), em, childItemLabel);

                        /**
                         * CUSTOM ATTRIBUTE
                         */
                        ExportUtility.insertCustomAttribute(em, childitem, childItemLabel, childitemLocalizationList.get(0).getLanguage());

                        arrayListItemLabel.add(childItemLabel);
                    }
                }

                /**
                 * for the codelist set the collection and the register
                 */
                CollectionItemLabel collectionLabelList = new CollectionItemLabel();
                collectionLabelList.setItemLabel(arrayListItemLabel);
                itemLabel.setCollection(collectionLabelList);

            }
        }
        return itemFile;
    }

    private void createXML(File itemFile, ItemLabel itemLabel) {
        File completeCodelistNameXML = new File(itemFile.getAbsolutePath().concat(folderSeparator + itemLabel.getUri_name() + "." + itemLabel.getLabel().getLanguage() + ".xml"));
        try {
            /**
             * write the XML file
             */
            JAXBContext jaxbContextLang = JAXBContext.newInstance(ItemLabel.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            marshallerclassLanguage.marshal(itemLabel, completeCodelistNameXML);
            log.info("Create with success the file " + completeCodelistNameXML);
        } catch (JAXBException ex) {
            log.error("Error in creation file: " + completeCodelistNameXML + " -JAXBException- " + ex.getMessage());
        }
    }

    private void setCodelist(Item itemCollection, EntityManager em, Languagecode localizationLanguage, ItemLabel itemLabel) {
        if (itemCollection != null) {
            String codeListUriName = itemCollection.getUriName();

            String PARENTITEM_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.item = :itemCollection  AND loc.language = :localizationLanguage";
            Query parentlc = em.createQuery(PARENTITEM_LOCALIZATION_QUERY);
            parentlc.setParameter("itemCollection", itemCollection);
            parentlc.setParameter("localizationLanguage", localizationLanguage);
            String parentItemLabel = ((Localization) parentlc.getResultList().get(0)).getLabel();

            CodelistAttribute codelistAttribute = new CodelistAttribute();
            codelistAttribute.setUri_name(codeListUriName);

            LabelAttribute labelAttribute = new LabelAttribute();
            labelAttribute.setValue(parentItemLabel);
            labelAttribute.setLanguage(localizationLanguage.getIsocode());

            codelistAttribute.setLabel(labelAttribute);

            itemLabel.setCodelist(codelistAttribute);
        }
    }

    private void setRegister(Register itemRegister, EntityManager em, Languagecode localizationLanguage, ItemLabel itemLabel) {
        String registerBaseUri = itemRegister.getBaseUri();
        String registerUriName = itemRegister.getUriName();

        RegisterAttribute registerAttribute = new RegisterAttribute();
        registerAttribute.setBase_uri(registerBaseUri);
        registerAttribute.setUri_name(registerUriName);

        String REGISTER_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.register = :register AND loc.language = :localizationLanguage";
        Query qrel = em.createQuery(REGISTER_LOCALIZATION_QUERY);
        qrel.setParameter("register", itemRegister);
        qrel.setParameter("localizationLanguage", localizationLanguage);

        List<Localization> localizationRegisterList = qrel.getResultList();
        for (Localization registerlocalization : localizationRegisterList) {
            LabelAttribute registerlabelAttribute = new LabelAttribute();
            registerlabelAttribute.setValue(registerlocalization.getLabel());
            registerlabelAttribute.setLanguage(registerlocalization.getLanguage().getIsocode());

            registerAttribute.setLabel(registerlabelAttribute);
        }

        RegistryAttribute registryAttribute = new RegistryAttribute();
        Registry registry = itemRegister.getRegistry();
        registryAttribute.setBase_uri(registry.getBaseUri());
        registryAttribute.setUri_name(registry.getUriName());

        String REGISTRY_LOCALIZATION_QUERRY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.registry = :registry AND loc.language = :registrylocalizationLanguage";
        Query rlq = em.createQuery(REGISTRY_LOCALIZATION_QUERRY);
        rlq.setParameter("registry", registry);
        rlq.setParameter("registrylocalizationLanguage", localizationLanguage);

        List<Localization> registryLocalization = rlq.getResultList();
        for (Localization registerLocalizationItem : registryLocalization) {
            String registerLocalizationItemLabel = registerLocalizationItem.getLabel();

            LabelAttribute labelAttribute = new LabelAttribute();
            labelAttribute.setValue(registerLocalizationItemLabel);
            labelAttribute.setLanguage(registerLocalizationItem.getLanguage().getIsocode());

            registryAttribute.setLabel(labelAttribute);
        }

        registerAttribute.setRegistry(registryAttribute);

        itemLabel.setRegister(registerAttribute);
    }

    private void insertExternalReferenceCodelist(ExternalReference externalReference, ItemLabel itemLabel, File exportFile) {
        if (externalReference != null) {
            File referenceFile = new File(exportFile.getAbsolutePath().replace(registerLabel.getUri_name(), "reference"));
            if (!referenceFile.exists()) {
                referenceFile.mkdir();
            }

            Query q = em.createNamedQuery("ExternalReference.findByUuid");
            q.setParameter("uuid", externalReference.getUuid());

            List<ExternalReference> externalReferenceList = q.getResultList();
            if (!externalReferenceList.isEmpty()) {
                for (ExternalReference externalReferenceItem : externalReferenceList) {
                    ExternalReferenceAttribute externalReferenceAttribute = new ExternalReferenceAttribute();

                    externalReferenceAttribute.setBase_uri(itemLabel.getRegister().getBase_uri());
                    externalReferenceAttribute.setUri_name(itemLabel.getUri_name());
                    externalReferenceAttribute.setLanguage(itemLabel.getLabel().getLanguage());

                    externalReferenceAttribute.setRegister(itemLabel.getRegister());
                    
                    CodelistAttribute codelistAttribute = new CodelistAttribute();
                    codelistAttribute.setUri_name(itemLabel.getUri_name());
                    codelistAttribute.setLabel(itemLabel.getLabel());
                    externalReferenceAttribute.setCodelist(codelistAttribute);

                    externalReferenceAttribute.setAuthoritativesource(externalReferenceItem.getAuthoritativesource());
                    externalReferenceAttribute.setGovernancebody(externalReferenceItem.getGovernancebody());
                    externalReferenceAttribute.setAvailablelanguages(externalReferenceItem.getAvailablelanguages());
                    externalReferenceAttribute.setInformativelink(externalReferenceItem.getInformativelink());
                    itemLabel.setExternalreference(externalReferenceAttribute);

                    File externalreferenceFile = new File(referenceFile.getAbsoluteFile() + folderSeparator + itemLabel.getUri_name());
                    if (!externalreferenceFile.exists()) {
                        externalreferenceFile.mkdir();
                    }

                    File externalreferenceCodelistNameXML = new File(externalreferenceFile.getAbsolutePath().concat(folderSeparator + itemLabel.getUri_name() + "." + itemLabel.getLabel().getLanguage() + ".xml"));
                    try {
                        /**
                         * write the XML file
                         */
                        JAXBContext jaxbContextLang = JAXBContext.newInstance(ExternalReferenceAttribute.class);
                        Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
                        marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

                        marshallerclassLanguage.marshal(externalReferenceAttribute, externalreferenceCodelistNameXML);
                        log.info("Create with success the file " + externalreferenceCodelistNameXML);
                    } catch (JAXBException ex) {
                        log.error("Error in creation file: " + externalreferenceCodelistNameXML + " -JAXBException- " + ex.getMessage());
                    }
                }
            }
        }
    }
}