/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 14/06/2013
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.re3gistryexport.marshaller;

import eu.europa.ec.re3gistry.constants.EntityNames;
import eu.europa.ec.re3gistry.jaxbmodel.register.ArrayRegisterLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.ArrayRegistryLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.CollectionRegisterLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.CollectionRegistryLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.RegisterLabel;
import eu.europa.ec.re3gistry.jaxbmodel.register.RegistersLabelList;
import eu.europa.ec.re3gistry.jaxbmodel.register.RegistryLabel;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.LabelAttribute;
import eu.europa.ec.re3gistry.jaxbmodel.utilattribute.RegistryAttribute;
import eu.europa.ec.re3gistry.model.Item;
import eu.europa.ec.re3gistry.model.Languagecode;
import eu.europa.ec.re3gistry.model.Localization;
import eu.europa.ec.re3gistry.model.Register;
import eu.europa.ec.re3gistry.model.Registry;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;

public class SolrExport {

    private String folderSeparator = System.getProperty("file.separator");
    private static org.apache.log4j.Logger log = org.apache.log4j.Logger.getLogger(SolrExport.class);

    public SolrExport(String folderExport, EntityManager em, List<Registry> registryList) {

        /**
         * create the REGISTERS folder
         */
        File fileRegistersFolder = new File(folderExport);
        if (!fileRegistersFolder.exists()) {
            fileRegistersFolder.mkdir();
        }
        /**
         * create registry files
         */
        List<Register> registerList = createRegistryXMLFiles(fileRegistersFolder, em);
        log.info("There are available " + registerList.size() + " registers");
        /**
         * for each register create the complete file and the folder for each
         * codelist and values
         */
        for (Register register : registerList) {
            log.info(" === Start the creation of the register folder ===");
            createRegisterFolder(fileRegistersFolder, em, register);
            log.info(" === End creation of the register === ");
        }
    }

    private List<Register> createRegistryXMLFiles(File fileRegistersFolder, EntityManager em) {
        List<Register> registerList = null;
        /**
         * get all the registry
         */
        Query qregistry = em.createNamedQuery("Registry.findAll");
        List<Registry> registryList = qregistry.getResultList();
        log.info("There are " + registryList.size() + " registry.");
        /**
         * for each registry
         */
        ArrayList<RegistryLabel> arrayRegistryLabel = new ArrayList<RegistryLabel>();
        ArrayRegistryLabel arrayRegistryLabelClass = new ArrayRegistryLabel();
        CollectionRegistryLabel collectionRegistryLabel = new CollectionRegistryLabel();

        for (Registry registry : registryList) {
            /**
             * registry details
             */
            String registryUUID = registry.getUuid();
            String registryBaseUri = registry.getBaseUri();
            String registryUriName = registry.getUriName();
            Date registryDatecreation = registry.getDatecreation();
            Date registryDatelastupdate = registry.getDatelastupdate();
            String registryRegistrynameger = registry.getRegistrymanager();

            /**
             * get the localization for the registry
             */
            String REGISTRY_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.registry = :registry";
            Query localisationRegistry = em.createQuery(REGISTRY_LOCALIZATION_QUERY);
            localisationRegistry.setParameter("registry", registry);

            /**
             * for every language of the registry
             */
            List<Localization> localizationRegistryList = localisationRegistry.getResultList();

            for (Localization localizationRegistry : localizationRegistryList) {
                /**
                 * create the RegistryLabel with all the detail of the registry
                 */
                RegistryLabel registryLabel = new RegistryLabel();
                registryLabel.setUri_name(registryUriName);
                registryLabel.setBase_uri(registryBaseUri);
                registryLabel.setDatecreation(registryDatecreation);
                registryLabel.setDatelastupdate(registryDatelastupdate);
                registryLabel.setRegistrymanager(registryRegistrynameger);

                String label = localizationRegistry.getLabel();
                Languagecode registryLanguage = localizationRegistry.getLanguage();

                log.info("Creation of the registry with the label " + registryLabel + " for the language " + registryLanguage.getIsocode());

                String definition = localizationRegistry.getDefinition();
                String description = localizationRegistry.getDescription();
                String registryIsocode = registryLanguage.getIsocode();

                registryLabel.setLabel(label, registryIsocode);
                registryLabel.setDefinition(definition, registryIsocode);
                registryLabel.setDescription(description, registryIsocode);

                /**
                 * get all the register for the specific registry
                 */
                String REGISTRER_LOCALIZATION_QUERY = "SELECT register FROM " + EntityNames.ENTITY_REGISTER + " register WHERE register.registry = :registry ORDER BY register.uuid";
                Query localisationRegister = em.createQuery(REGISTRER_LOCALIZATION_QUERY);
                localisationRegister.setParameter("registry", registry);
                registerList = localisationRegister.getResultList();

                ArrayList<RegisterLabel> arrayList = new ArrayList<RegisterLabel>();

                /**
                 * for each register
                 */
                for (Register register : registerList) {
                    /**
                     * register details
                     */
                    String registerUUID = register.getUuid();
                    String registerBaseUri = register.getBaseUri();
                    String registerUriName = register.getUriName();
                    Date registerDatecreation = register.getDatecreation();
                    Date registerDatelastupdate = register.getDatelastupdate();
                    String registerOwner = register.getRegisterowner();
                    String registerManeger = register.getRegistermanager();
                    String registerControlBudy = register.getRegistercontrolbody();
                    Registry registrerRegistry = register.getRegistry();

                    /**
                     * create the RegisterLabel for each register
                     */
                    RegisterLabel registerLabel = new RegisterLabel();
                    registerLabel.setUuid(registerUUID);
                    registerLabel.setBase_uri(registerBaseUri);
                    registerLabel.setUri_name(registerUriName);
                    registerLabel.setDatecreation(registerDatecreation);
                    registerLabel.setDatelastupdate(registerDatelastupdate);
                    registerLabel.setOwner(registerOwner);
                    registerLabel.setManager(registerManeger);
                    registerLabel.setControlbody(registerControlBudy);
                    /**
                     * get the localization for the register
                     */
                    String REGISTER_LOCALIZATION_QUERY = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.register = :register AND loc.language = :registryLanguage";
                    Query qrel = em.createQuery(REGISTER_LOCALIZATION_QUERY);
                    qrel.setParameter("register", register);
                    qrel.setParameter("registryLanguage", registryLanguage);
                    List<Localization> registerLocalization = qrel.getResultList();

                    /**
                     * for each localization of the register
                     */
                    for (Localization registerLocalizationItem : registerLocalization) {
                        String registerLocalizationItemLabel = registerLocalizationItem.getLabel();

                        registerLabel.setLabel(registerLocalizationItemLabel, registryLanguage.getIsocode());
                    }
                    /**
                     * add the registerLabel to an array
                     */
                    arrayList.add(registerLabel);
                }

                RegistersLabelList registersLabelList = new RegistersLabelList();
                registersLabelList.setRegisterLabel(arrayList);

                registryLabel.setRegistryLabelList(registersLabelList);

                arrayRegistryLabel.add(registryLabel);
            }
        }
        collectionRegistryLabel.setRegistryLabel(arrayRegistryLabel);
        arrayRegistryLabelClass.setCollectionRegistryLabel(collectionRegistryLabel);

        try {
            JAXBContext jaxbContextLang = JAXBContext.newInstance(ArrayRegistryLabel.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            System.out.print("fileRegistersFolder.getAbsolutePath(): " + fileRegistersFolder.getAbsolutePath() + "---con: " + fileRegistersFolder.getAbsolutePath().concat(folderSeparator + "completed.xml"));
            File completeRegistryNameXML = new File(fileRegistersFolder.getAbsolutePath().concat(folderSeparator + "registry.xml"));
            marshallerclassLanguage.marshal(arrayRegistryLabelClass, completeRegistryNameXML);
        } catch (JAXBException ex) {
            Logger.getLogger(SolrExport.class.getName()).log(Level.SEVERE, null, ex);
        }

        return registerList;
    }

    private void createRegisterFolder(File fileRegistersFolder, EntityManager em, Register register) {
        try {

            ArrayList<RegisterLabel> arrayRegisterLabel = new ArrayList<RegisterLabel>();
            ArrayRegisterLabel arrayRegisterLabelClass = new ArrayRegisterLabel();
            CollectionRegisterLabel collectionRegisterLabel = new CollectionRegisterLabel();
            /**
             * register details
             */
            String registerUUID = register.getUuid();
            String registerBaseUri = register.getBaseUri();
            String registerUriName = register.getUriName();
            Date registerDatecreation = register.getDatecreation();
            Date registerDatelastupdate = register.getDatelastupdate();
            String registerOwner = register.getRegisterowner();
            String registerManeger = register.getRegistermanager();
            String registerControlBudy = register.getRegistercontrolbody();
            Registry registrerRegistry = register.getRegistry();

            /**
             * create register folder
             */
            String generateRegisterFolder = fileRegistersFolder.getAbsolutePath().concat(folderSeparator + registerUriName);
            File fileRegisterFolder = new File(generateRegisterFolder);
            if (!fileRegisterFolder.exists()) {
                fileRegisterFolder.mkdir();
            }

            /**
             * get the items for that register
             */
            String REGISTER_ITEM_QUERY = "SELECT it FROM " + EntityNames.ENTITY_ITEM + " it WHERE it.register = :register";
            Query qri = em.createQuery(REGISTER_ITEM_QUERY);
            qri.setParameter("register", register);

            List<Item> itemsList = qri.getResultList();
            log.info("*** Start the creation of the codelist and codelistvalue for the register" + registerUriName + " ***");
            new SolrCodelistExport(generateRegisterFolder, em, itemsList);
            log.info("***End with creation of the codelist and codelistvalue for the register" + registerUriName + " ***");

            /**
             * for each localization of the register
             */
            String REGISTER_LOCALIZATION_QUERY = "SELECT it FROM " + EntityNames.ENTITY_LOCALIZATION + " it WHERE it.register = :register";
            Query qrl = em.createQuery(REGISTER_LOCALIZATION_QUERY);
            qrl.setParameter("register", register);
            List<Localization> registerLocalization = qrl.getResultList();

            for (Localization registerLocalizationItem : registerLocalization) {
                String registerLocalizationItemLabel = registerLocalizationItem.getLabel();
                String localizationDefinition = registerLocalizationItem.getDefinition();
                String localizationDescription = registerLocalizationItem.getDescription();
                Date localizationDatecreation = registerLocalizationItem.getDatecreation();
                Date localizationDatelastmodification = registerLocalizationItem.getDatelastupdate();

                Languagecode registerLanguage = registerLocalizationItem.getLanguage();

                log.info("Create the register with the label " + registerLocalizationItemLabel + " for the language " + registerLanguage.getIsocode());

                /**
                 * create the RegisterLabel for each register
                 */
                RegisterLabel registerLabel = new RegisterLabel();
                registerLabel.setUuid(registerUUID);
                registerLabel.setBase_uri(registerBaseUri);
                registerLabel.setUri_name(registerUriName);
                registerLabel.setDatecreation(registerDatecreation);
                registerLabel.setDatelastupdate(registerDatelastupdate);
                registerLabel.setOwner(registerOwner);
                registerLabel.setManager(registerManeger);
                registerLabel.setControlbody(registerControlBudy);

                registerLabel.setLabel(registerLocalizationItemLabel, registerLanguage.getIsocode());
                registerLabel.setDefinition(localizationDefinition, registerLanguage.getIsocode());
                registerLabel.setDescription(localizationDescription, registerLanguage.getIsocode());

                /**
                 * get the localization for the register
                 */
                String REGISTRY_LOCALIZATION = "SELECT loc FROM " + EntityNames.ENTITY_LOCALIZATION + " loc WHERE loc.registry = :registrerRegistry and loc.language = :registerLanguage";
                Query qr = em.createQuery(REGISTRY_LOCALIZATION);
                qr.setParameter("registrerRegistry", registrerRegistry);
                qr.setParameter("registerLanguage", registerLanguage);

                RegistryAttribute registryAttribute = new RegistryAttribute();
                registryAttribute.setBase_uri(registrerRegistry.getBaseUri());
                registryAttribute.setUri_name(registrerRegistry.getUriName());

                List<Localization> registryLocalization = qr.getResultList();
                for (Localization registryLocalizationItem : registryLocalization) {
                    log.info("The register " + registerLabel + " contains " + registryLocalization.size() + " items.");

                    String registryLocalizationItemLabel = registryLocalizationItem.getLabel();
                    Languagecode registryLocalizationItemLanguage = registryLocalizationItem.getLanguage();
                    if (registerLanguage.equals(registryLocalizationItemLanguage)) {

                        LabelAttribute labelAttribute = new LabelAttribute();
                        labelAttribute.setValue(registryLocalizationItemLabel);
                        labelAttribute.setLanguage(registryLocalizationItemLanguage.getIsocode());

                        registryAttribute.setLabel(labelAttribute);
                    }
                }

                registerLabel.setRegistry(registryAttribute);
                arrayRegisterLabel.add(registerLabel);
            }
            collectionRegisterLabel.setRegisterLabel(arrayRegisterLabel);
            arrayRegisterLabelClass.setCollectionRegisterLabel(collectionRegisterLabel);

            JAXBContext jaxbContextLang = JAXBContext.newInstance(ArrayRegisterLabel.class);
            Marshaller marshallerclassLanguage = jaxbContextLang.createMarshaller();
            marshallerclassLanguage.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

            File completeRegisterNameXML = new File(fileRegisterFolder.getAbsolutePath().concat(folderSeparator + registerUriName + ".xml"));
            marshallerclassLanguage.marshal(arrayRegisterLabelClass, completeRegisterNameXML);

            log.info("Created with success the file " + completeRegisterNameXML);
        } catch (JAXBException ex) {
            Logger.getLogger(SolrExport.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}