/*
* Copyright 2010,2013 EUROPEAN UNION
* Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* http://ec.europa.eu/idabc/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* Date: 14/06/2013
* Authors: European Commission, Joint Research Centre
* Daniele Francioli, Emanuela Epure
* inspire-registry-info@jrc.ec.europa.eu
*
* This work was supported by the EU  Interoperability Solutions for
* European Public Administrations Programme (http://ec.europa.eu/isa)
* through Action 1.17: Re-usable INSPIRE Reference Platform 
* (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
*/

package eu.europa.ec.re3gistry.re3gistryexport.servlet;

import eu.europa.ec.re3gistry.re3gistryexport.manager.Configuration;
import eu.europa.ec.re3gistry.re3gistryexport.xsltrasformations.TransformerMgr;
import java.io.File;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;

public class TransformationServlet extends HttpServlet {

    /**
     * Main folder properties
     */
    private static final String KEY_EXPORT_MAINFOLDER_CUSTOM = "export.mainfolder.custom";
    private static final String KEY_EXPORT_MAINFOLDER_NAME = "export.mainfolder.name";
    private static final String KEY_EXPORT_STANDARD_FOLDER_NAME = "export.folder.standard";
    private static final String KEY_EXPORT_CUSTOM_FOLDER_NAME = "export.folder.custom";
    private static final String KEY_EXPORT_XSL_MAINFOLDER_CUSTOM = "xsl.mainfolder.custom";
    private static final String KEY_EXPORT_XSL_MAINFOLDER = "xsl.mainfolder";
    private static final String KEY_EXPORT_FOLDERIDENTIFIER = "export.context.folderidentifier";
    
    /**
     * Format identifier list
     */
    private static final String KEY_EXPORT_XSL_XML_LIST = "xsl.xml.list";
    private static final String KEY_EXPORT_FORMAT_AVAILABLE = "transformation.available";
    private static final String KEY_EXPORT_CUSTOMTRANSFORMATION_AVAILABLE = "transformations.custom.available";
    
    /**
     * Specific properties values
     */
    private static final String VALUE_XSL_FILEMNAME_BASE_PROPERTY = "xsl.xml.filename.";
    private static final String VALUE_XSL_FOLDERNAME_BASE_PROPERTY = "xsl.xml.folderin.";
    private static final String VALUE_XSL_RECURSIVE_BASE_PROPERTY = "xsl.xml.recursive.";
    private static final String VALUE_XSL_FOLDERDEPTH_BASE_PROPERTY = "xsl.xml.folderdepth.";
    private static final String VALUE_TRANSFORMATION_FILEMNAME_BASE_PROPERTY = "transformation.xsl.filename.";
    private static final String VALUE_TRANSFORMATION_FOLDERNAME_BASE_PROPERTY = "transformation.xsl.folderin.";
    private static final String VALUE_TRANSFORMATION_STARTFROM_BASE_PROPERTY = "transformation.xsl.startfrom.";
    private static final String VALUE_TRANSFORMATION_OUTFILEEXTENSION_BASE_PROPERTY = "transformation.xsl.outfileextension.";
    private static final String VALUE_TRANSFORMATION_RECURSIVE_BASE_PROPERTY = "transformation.xsl.recursive.";
    private static final String VALUE_TRANSFORMATION_FOLDERDEPTH_BASE_PROPERTY = "transformation.xsl.folderdepth.";
    private static final String VALUE_CUSTOMTRANSFORMATIONN_XSLPATH = "transformations.custom.xslpath.";
    private static final String VALUE_CUSTOMTRANSFORMATION_OUTFILEFOLDER = "transformations.custom.outfilefolder.";
    private static final String VALUE_CUSTOMTRANSFORMATION_OUTFILEFORMAT = "transformations.custom.outfileformat.";
    private static final String VALUE_CUSTOMTRANSFORMATION_INFOLDER = "transformations.custom.infolder.";

   
    /**
     * XML file extension value
     */
    private static final String VALUE_XML_FILE_EXTENSION = "xml";
    
    /**
     * Common values
     */
    private static final String VALUE_STARTFROM_CUSTOM = "custom";
    private static final String VALUE_STARTFROM_STANDARD = "standard";
    
    /**
     * Creating logger
     */
    private static Logger log = Logger.getLogger(TransformationServlet.class);

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        try {
            /**
             * Getting properties
             * 
             * Main export folder properties
             */
            String exportMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_CUSTOM, null);
            String exportMainfolderName = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_NAME, "export");
            String exportStandardFolder = Configuration.getInstance().getProperty(KEY_EXPORT_STANDARD_FOLDER_NAME, "standard");
            String exportCustomFolder = Configuration.getInstance().getProperty(KEY_EXPORT_CUSTOM_FOLDER_NAME, "custom");
            exportMainfolderCustom = (exportMainfolderCustom != null) ? exportMainfolderCustom.trim() : null;
            exportMainfolderName = (exportMainfolderName != null) ? exportMainfolderName.trim() : null;
            exportStandardFolder = (exportStandardFolder != null) ? exportStandardFolder.trim() : null;

            /**
             * XSL Properties
             */
            String xslMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_XSL_MAINFOLDER_CUSTOM, null);
            String xslMainfolder = Configuration.getInstance().getProperty(KEY_EXPORT_XSL_MAINFOLDER, "xsl.mainfolder");
            String xslXmlList = Configuration.getInstance().getProperty(KEY_EXPORT_XSL_XML_LIST, null);
            xslMainfolderCustom = (xslMainfolderCustom != null) ? xslMainfolderCustom.trim() : null;
            xslMainfolder = (xslMainfolder != null) ? xslMainfolder.trim() : null;
            xslXmlList = (xslXmlList != null) ? xslXmlList.trim() : null;

            /**
             * Other formats properties
             */
            String exportAvailableFormats = Configuration.getInstance().getProperty(KEY_EXPORT_FORMAT_AVAILABLE, "");
            exportAvailableFormats = (exportAvailableFormats != null) ? exportAvailableFormats.trim() : null;
            
            /**
             * Custom configuration transformations properties
             */
            String customTransformationsAvailable = Configuration.getInstance().getProperty(KEY_EXPORT_CUSTOMTRANSFORMATION_AVAILABLE, "");
            customTransformationsAvailable = (customTransformationsAvailable != null) ? customTransformationsAvailable.trim() : null;
            

            /**
             * Instantiating other common variables
             */
            String contextFolderIdentifier = Configuration.getInstance().getProperty(KEY_EXPORT_FOLDERIDENTIFIER, "WEB-INF");
            contextFolderIdentifier = (contextFolderIdentifier != null) ? contextFolderIdentifier.trim() : null;
            String relativePath = getServletContext().getRealPath(contextFolderIdentifier);
            String folderSeparator = System.getProperty("file.separator");

            /**
             * Initializing the transformation manager
             */
            TransformerMgr t = new TransformerMgr(getServletContext());

            /**
             * Getting base DB export folder
             */
            String folderDbExport = null;
            if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                folderDbExport = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + exportStandardFolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                folderDbExport = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + exportStandardFolder;
            }

            //Getting base xsl folder*/
            String folderXsl = null;
            if (xslMainfolderCustom != null && !xslMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                folderXsl = xslMainfolderCustom + folderSeparator + xslMainfolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                folderXsl = relativePath.replaceAll(contextFolderIdentifier, "") + xslMainfolder;
            }

            // ## XML transformations processing ## //
            if (folderDbExport != null && !folderDbExport.trim().equals("") && folderXsl != null && !folderXsl.trim().equals("")) {
                /**
                 * Getting all xml transformation modes
                 */
                String[] mode = xslXmlList.split("-");
                for (int i = 0; i < mode.length; i++) {
                    /**
                     * Getting specific mode properties
                     */
                    String currentXslFilename = Configuration.getInstance().getProperty(VALUE_XSL_FILEMNAME_BASE_PROPERTY + mode[i], null);
                    String currentInSpecificFolder = Configuration.getInstance().getProperty(VALUE_XSL_FOLDERNAME_BASE_PROPERTY + mode[i], null);
                    String recursiveFlag = Configuration.getInstance().getProperty(VALUE_XSL_RECURSIVE_BASE_PROPERTY + mode[i], null);
                    String folderDepth = Configuration.getInstance().getProperty(VALUE_XSL_FOLDERDEPTH_BASE_PROPERTY + mode[i], null);
                    currentXslFilename = (currentXslFilename != null) ? currentXslFilename.trim() : null;
                    currentInSpecificFolder = (currentInSpecificFolder != null) ? currentInSpecificFolder.trim() : "";
                    recursiveFlag = (recursiveFlag != null) ? recursiveFlag.trim() : null;
                    folderDepth = (folderDepth != null) ? folderDepth.trim() : null;

                    /**
                     * Creating complete file and folder names
                     */
                    String xslFile = folderXsl + folderSeparator + currentXslFilename;
                    String inFolder = folderDbExport + folderSeparator + currentInSpecificFolder;

                    try {
                        File tmp = new File(xslFile);
                        if (tmp != null && !tmp.isFile()) {
                            log.info("An error occurred while loading files or folder: cannot load xsl file");
                            continue;
                        }
                        tmp = new File(inFolder);
                        if (tmp != null && !tmp.isDirectory()) {
                            log.info("An error occurred while loading files or folder: cannot ");
                            continue;
                        }
                    } catch (Exception e) {
                        log.info("An error occurred while loading files or folder.", e);
                        continue;
                    }

                    /**
                     * Launching the correct transformation engine
                     */
                    if (recursiveFlag != null && recursiveFlag.trim().equals("false")) {
                        log.info("Transformation started; folder: "+inFolder+" - xsl: "+xslFile);
                        t.transform(new File(inFolder), xslFile, VALUE_XML_FILE_EXTENSION, getServletContext());
                        log.info("Transformation ended");
                    } else if (recursiveFlag != null && recursiveFlag.trim().equals("true")) {
                        int md = 0;
                        if (folderDepth != null && !folderDepth.trim().equals("")) {
                            try {
                                md = Integer.parseInt(folderDepth);
                            } catch (Exception e) {
                                log.info("An error occurred while reading max depth properties.");
                                continue;
                            }
                        } else {
                            log.info("An error occurred while reading max depth properties: no properties setted");
                            continue;
                        }
                        log.info("Transformation (recursive) started; folder: "+inFolder+" - xsl: "+xslFile);
                        t.transformRecursively(new File(inFolder), xslFile, VALUE_XML_FILE_EXTENSION, md, getServletContext());
                        log.info("Transformation ended");
                    } else {
                    }
                }
            } else {
                log.info("An error occurred while reading folders, the task could not be performed.");
            }


            // ## Other formats transformations processing ## //
            /**
             * Getting custom export folder
             */
            String customExportFolder = null;
            if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                customExportFolder = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + exportCustomFolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                customExportFolder = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + exportCustomFolder;
            }

            if (customExportFolder != null && !customExportFolder.trim().equals("") && folderXsl != null && !folderXsl.trim().equals("")) {
                /**
                 * Getting all xml transformation modes
                 */
                String[] mode = exportAvailableFormats.split("-");
                for (int i = 0; i < mode.length; i++) {
                    /**
                     * Getting specific mode properties
                     */
                    String currentXslFilename = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FILEMNAME_BASE_PROPERTY + mode[i], null);
                    String currentInSpecificFolder = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FOLDERNAME_BASE_PROPERTY + mode[i], null);
                    String startFrom = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_STARTFROM_BASE_PROPERTY + mode[i], null);
                    String recursiveFlag = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_RECURSIVE_BASE_PROPERTY + mode[i], null);
                    String folderDepth = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FOLDERDEPTH_BASE_PROPERTY + mode[i], null);
                    String outFileExtension = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_OUTFILEEXTENSION_BASE_PROPERTY + mode[i], null);
                    currentXslFilename = (currentXslFilename != null) ? currentXslFilename.trim() : null;
                    currentInSpecificFolder = (currentInSpecificFolder != null) ? currentInSpecificFolder.trim() : "";
                    recursiveFlag = (recursiveFlag != null) ? recursiveFlag.trim() : null;
                    folderDepth = (folderDepth != null) ? folderDepth.trim() : null;
                    outFileExtension = (outFileExtension != null) ? outFileExtension.trim() : null;

                    /**
                     * Creating complete file and folder names
                     */
                    String xslFile = folderXsl + folderSeparator + currentXslFilename;
                    String inFolder = null;
                    if (startFrom.trim().equals(VALUE_STARTFROM_CUSTOM)) {
                        inFolder = customExportFolder + folderSeparator + currentInSpecificFolder;
                    } else if (startFrom.trim().equals(VALUE_STARTFROM_STANDARD)) {
                        inFolder = folderDbExport + folderSeparator + currentInSpecificFolder;
                    }

                    try {
                        File tmp = new File(xslFile);
                        if (tmp != null && !tmp.isFile()) {
                            log.info("An error occurred while loading files or folder: cannot load xsl file");
                            continue;
                        }
                        tmp = new File(inFolder);
                        if (tmp != null && !tmp.isDirectory()) {
                            log.info("An error occurred while loading files or folder: cannot ");
                            continue;
                        }
                    } catch (Exception e) {
                        log.info("An error occurred while loading files or folder.", e);
                        continue;
                    }

                    /**
                     * Launching the correct transformation engine
                     */
                    if (recursiveFlag != null && recursiveFlag.trim().equals("false")) {
                        log.info("Transformation started; folder: "+inFolder+" - xsl: "+xslFile);
                        t.transform(new File(inFolder), xslFile, outFileExtension, getServletContext());
                        log.info("Transformation ended");
                    } else if (recursiveFlag != null && recursiveFlag.trim().equals("true")) {
                        int md = 0;
                        if (folderDepth != null && !folderDepth.trim().equals("")) {
                            try {
                                md = Integer.parseInt(folderDepth);
                            } catch (Exception e) {
                                log.info("An error occurred while reading max depth properties.");
                                continue;
                            }
                        } else {
                            log.info("An error occurred while reading max depth properties: no properties setted");
                            continue;
                        }
                        log.info("Transformation (recursive) started; folder: "+inFolder+" - xsl: "+xslFile);
                        t.transformRecursively(new File(inFolder), xslFile, outFileExtension, md, getServletContext());
                        log.info("Transformation ended");
                    } else {
                    }
                }
            } else {
                log.info("An error occurred while reading folders, the task could not be performed.");
            }
            
            
            /*Custom configurations transformations*/
            /**
             * Getting custom export folder
             */
            String transformationCustomExportFolder = null;
            if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                transformationCustomExportFolder = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + exportCustomFolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                transformationCustomExportFolder = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + exportCustomFolder;
            }

            if (transformationCustomExportFolder != null && !transformationCustomExportFolder.trim().equals("")) {
                /**
                 * Getting all xml custom configuration transformation modes
                 */
                String[] mode = customTransformationsAvailable.split("-");
                for (int i = 0; i < mode.length; i++) {
                    /**
                     * Getting specific mode properties
                     */
                    String xslPath = Configuration.getInstance().getProperty(VALUE_CUSTOMTRANSFORMATIONN_XSLPATH + mode[i], null);
                    String inFolder = Configuration.getInstance().getProperty(VALUE_CUSTOMTRANSFORMATION_INFOLDER + mode[i], null);
                    String outFolder = Configuration.getInstance().getProperty(VALUE_CUSTOMTRANSFORMATION_OUTFILEFOLDER + mode[i], null);
                    String outFormat = Configuration.getInstance().getProperty(VALUE_CUSTOMTRANSFORMATION_OUTFILEFORMAT + mode[i], null);
                    xslPath = (xslPath != null) ? xslPath.trim() : null;
                    inFolder = (inFolder != null) ? inFolder.trim() : "";
                    outFolder = (outFolder != null) ? outFolder.trim() : null;
                    outFormat = (outFormat != null) ? outFormat.trim() : null;


                    /**
                     * Creating complete file and folder names
                     */
                    String xslFile = folderXsl + folderSeparator + xslPath;

                    try {
                        File tmp = new File(xslFile);
                        if (tmp != null && !tmp.isFile()) {
                            log.info("An error occurred while loading files or folder: cannot load xsl file");
                            continue;
                        }
                        tmp = new File(inFolder);
                        if (tmp != null && !tmp.isDirectory()) {
                            log.info("An error occurred while loading files or folder: cannot ");
                            continue;
                        }
                    } catch (Exception e) {
                        log.info("An error occurred while loading files or folder.", e);
                        continue;
                    }

                    
                    t.transformCustom(new File(inFolder), xslFile, transformationCustomExportFolder, outFolder, outFormat, getServletContext());
                    
                }
            } else {
                log.info("An error occurred while reading folders, the task could not be performed.");
            }          
            
            response.sendRedirect("index.jsp");

        } catch (Exception e) {
            log.error("An error has occurred while executing the transformation job.", e);
        }
    }
}
